package br.com.furutani.dao;

import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Query;

import org.apache.log4j.Logger;
import org.hibernate.exception.ConstraintViolationException;

import br.com.furutani.entities.FeedRSS;
import br.com.furutani.entities.ItemRSS;
import br.com.furutani.jpa.EMUtil;

public class RSSDAOImpl implements RSSDAO {

	Logger log = Logger.getLogger(RSSDAOImpl.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.com.furutani.dao.RSSDAO#buscarFeedsCadastrados()
	 */
	public List<FeedRSS> buscarFeedsCadastrados() {

		List<FeedRSS> resultList = null;
		EntityManager entityManager = EMUtil.getEntityManager();

		resultList = entityManager.createQuery("from FeedRSS fr order by fr.ultimaLeitura asc").getResultList();

		return resultList;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * br.com.furutani.dao.RSSDAO#atualizar(br.com.furutani.entities.FeedRSS)
	 */
	public void atualizar(FeedRSS feed) {
		EntityManager entityManager = EMUtil.getEntityManager();
		EntityTransaction transaction = entityManager.getTransaction();

		try {
			transaction.begin();
			entityManager.merge(feed);
			transaction.commit();

			log.info("Commit " + feed.getUrl() + " Qtd de itens: " + feed.getItens().size());

		} catch (ConstraintViolationException e) {
			log.error("No Processou " + feed.getUrl());
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see br.com.furutani.dao.RSSDAO#cadastrar(java.util.List)
	 */
	public void cadastrar(List<FeedRSS> feeds) {
		EntityManager entityManager = EMUtil.getEntityManager();
		EntityTransaction transaction = entityManager.getTransaction();

		if (feeds == null) {
			return;
		}

		Iterator<FeedRSS> itr = feeds.iterator();

		while (itr.hasNext()) {
			try {
				transaction.begin();
				FeedRSS next = itr.next();
				entityManager.persist(next);
				transaction.commit();
				log.info("Cadastrou " + next.getUrl());
			} catch (Exception e) {
				log.error(e);
				transaction.rollback();
			}
		}
	}

	@Override
	public List<ItemRSS> buscarFeedsPeriodo(Calendar dataInicial, Calendar dataFinal) {
		List<ItemRSS> resultList = null;
		EntityManager entityManager = EMUtil.getEntityManager();

		Query query = entityManager.createQuery("from ItemRSS ir where ir.dataPublicao >= :dataIni and ir.dataPublicao <= :dataFim order by fr.ultimaLeitura asc");
		query.setParameter("dataIni", dataInicial);
		query.setParameter("dataFim", dataFinal);
		resultList = query.getResultList();

		return resultList;
	}
}
