package br.com.furutani.leitorrss;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import org.apache.log4j.Logger;
import org.hibernate.exception.ConstraintViolationException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import br.com.furutani.dao.RSSDAO;
import br.com.furutani.dao.RSSDAOImpl;
import br.com.furutani.entities.FeedRSS;
import br.com.furutani.entities.ItemRSS;

import com.tecnick.htmlutils.htmlentities.HTMLEntities;

/**
 * 
 * @author Roberto Jundi
 * @since 01/02/2009
 */
public class LeitorRSSService {
	private Logger log = Logger.getLogger(LeitorRSSService.class);
	private SimpleDateFormat sdf = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z", Locale.ENGLISH);

	
	/**
	 * Busca a lista de feeds cadastradas
	 * @return
	 */
	public List<FeedRSS> buscarFeeds() {
		RSSDAO dao = new RSSDAOImpl();
		List<FeedRSS> resultList = null;

		resultList = dao.buscarFeedsCadastrados();

		log.info("A pesquisa retornou: " + resultList.size());

		return resultList;
	}
	/**
	 * Busca a lista de feeds cadastradas
	 * @return
	 */
	public List<ItemRSS> buscarFeeds(Calendar dataInicial, Calendar dataFinal) {
		RSSDAO dao = new RSSDAOImpl();
		List<ItemRSS> resultList = null;

		resultList = dao.buscarFeedsPeriodo(dataInicial, dataFinal);

		log.info("A pesquisa retornou: " + resultList.size());

		return resultList;
	}

	/**
	 * Atualiza
	 * @param resultList
	 */
	public void atualizar(List<FeedRSS> resultList) {
		Iterator<FeedRSS> itrFeed = resultList.iterator();
		RSSDAO dao = new RSSDAOImpl();
		try {
			SAXBuilder builder = new SAXBuilder();

			while (itrFeed.hasNext()) {
				FeedRSS feed = itrFeed.next();

				try {
					java.net.URL url = new java.net.URL(feed.getUrl());
					url.openConnection();
					InputStream openStream = url.openStream();

					Document document = builder.build(openStream);
					Element root = document.getRootElement();

					Element child = root.getChild("channel");
					feed.setLink(HTMLEntities.htmlentities(child.getChildText("link")));
					feed.setCopyRight(HTMLEntities.htmlentities(child.getChildText("copyright")));
					feed.setDescricao(HTMLEntities.htmlentities(child.getChildText("description")));
					feed.setTitulo(HTMLEntities.htmlentities(child.getChildText("title")));

					List children = child.getChildren("item");
					Iterator itr = children.iterator();

					Set<ItemRSS> listaItens = feed.getItens();

					if (listaItens == null) {
						listaItens = new HashSet<ItemRSS>();
					}

					while (itr.hasNext()) {
						ItemRSS itemRSS = new ItemRSS(feed);
						Element e = (Element) itr.next();
						itemRSS.setTitulo(HTMLEntities.htmlentities(e.getChildText("title")));
						String link = HTMLEntities.htmlentities(e.getChildText("link"));
						
						// Verifica se o link no  maior que tamanho do campo no banco de dados
						if (link.length() > 255)
							link = link.substring(0, 250);
						itemRSS.setLink(link);
						
						itemRSS.setDescricao(HTMLEntities.htmlentities(e.getChildText("description")));
						try {
							String data = e.getChildText("pubDate");
							if (data != null){
								itemRSS.setDataPublicao(sdf.parse(data));
							} else {
								// Se no tiver a data de publicao, coloca a data atual
								itemRSS.setDataPublicao(new Date());
							}
						} catch (ParseException e1) {
							itemRSS.setDataPublicao(new Date());
						}
						listaItens.add(itemRSS);
					}
					feed.setItens(listaItens);
					feed.setUltimaLeitura(new Date());
					dao.atualizar(feed);

				} catch (UnknownHostException e) {
					log.error("Servidor no encontrado No Processou " + feed.getUrl());
				} catch (ConstraintViolationException e) {
					log.error("No Processou " + feed.getUrl());
				}
			}

		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (JDOMException e) {
			e.printStackTrace();
		}

	}

	/**
	 * Processa a URL informada no parametro para que seja cadastrada no
	 * banco de dados
	 * 
	 * @param url
	 *           URL do RSS 
	 */

	public void cadastrar(String url) {
		cadastrar(new String[] { url });
	}

	/**
	 * Processa as URLs informadas no parametro para que sejam cadastradas no
	 * banco de dados
	 * 
	 * @param urls
	 *            Array com as URLs
	 */
	public void cadastrar(String[] urls) {
		RSSDAO dao = new RSSDAOImpl();
		List<FeedRSS> lista = new ArrayList<FeedRSS>();
		SAXBuilder builder = new SAXBuilder();

		for (int idx = 0; idx < urls.length; idx++) {
			try {
				// Obtem o XML
				java.net.URL url = new java.net.URL(urls[idx]);
				url.openConnection();
				InputStream openStream = url.openStream();

				// Inicio do processamento do XML com jDOM
				Document document = builder.build(openStream);
				Element root = document.getRootElement();

				Element child = root.getChild("channel");
				
				// Cria um objeto FeedRSS
				FeedRSS feed = new FeedRSS();
				feed.setLink(HTMLEntities.htmlentities(child.getChildText("link")));
				feed.setCopyRight(HTMLEntities.htmlentities(child.getChildText("copyright")));
				feed.setDescricao(HTMLEntities.htmlentities(child.getChildText("description")));
				feed.setTitulo(HTMLEntities.htmlentities(child.getChildText("title")));
				feed.setUltimaLeitura(new Date());
				feed.setUrl(urls[idx]);
				lista.add(feed);

			} catch (MalformedURLException e) {
				log.error(e);
			} catch (IOException e) {
				log.error(e);
			} catch (JDOMException e) {
				log.error(e);
			} catch (Exception e) {
				log.error(e);
			}
		}
		// Cadastra a lista de feeds no banco
		dao.cadastrar(lista);
	}
}